#include <bios.h>

#define PutPixSafeAvg(x,y,c) if ((c3=GetPixel(x,y)) != 255) {\
                                if (c < 64) {\
                                   SetPixel(x,y,(c*2+c3)/3);\
                                } else {\
                                   SetPixel(x,y,(random(63)*2+c3)/3);\
                                }\
                             }

void CBlit_Scaled_String(char * string, int y, int c, int bgc,
                        bool trans, float xscale, float yscale) {
   Blit_Scaled_String(string, (MAXX-(strlen(string)*CHAR_WIDTH*xscale))/2, y, c, bgc,
                      trans, xscale, yscale);
   return;
}


void Make_Palette(void) {
   RGB_color temp;
   int c;

   temp.blue = 0;
   for (c=0; c < 64; c++) {
      temp.red = c;
      temp.green = c/3;
      Set_Palette_Register(c, temp);
   }
   
   temp.red = 0;
   temp.blue = 0;
   for (c=0; c < 64; c++) {
      temp.green = c;
      Set_Palette_Register(c+64, temp);
   }

   temp.red = 0;
   temp.green = 0;
   for (c=0; c < 64; c++) {
      temp.blue = c;
      Set_Palette_Register(c+128, temp);
   }

   for (c=0; c < 64 ; c++) {
      temp.red = temp.green = temp.blue = c;
      Set_Palette_Register(c+192, temp);
   }

   temp.red = 63;
   temp.green = 0;
   temp.blue = 0;
   Set_Palette_Register(255, temp);

   return;
}

void Flame(int textminx,  int textminy,  int textmaxx,  int textmaxy,
           int flameminx, int flameminy, int flamemaxx, int flamemaxy,
           bool hold) {
   int x,y;
   unsigned char c,c2,c3;
   
   for (y=flamemaxy; y > flameminy+1; y--) {
      for (x=flameminx; x < flamemaxx-1; x++) {
         c = GetPixel(x,y);
         
         if (x>=textminx && x<=textmaxx && y>=textminy && y<=(textmaxy+1)) {
            //If the text has been released, treat it as #63 so it
            //floats away.
            if (!hold) {
               if (c == 255) c = 63;
               if ((c2 = GetPixel(x,y-1)) == 255) c2 = 63;
               c2 = (c2 + c)/2;
               SetPixel(x,y-1,c2);
               PutPixSafeAvg(x-1,y-1,c2);
            } else {
               //If the current pixel is not text
               if (c != 255) {
                  //If the above pixel is not text
                  if ((c2 = GetPixel(x,y-1)) != 255) {
                     c2 = (c + c2)/2;
                     SetPixel(x,y-1,c2);
                  }
                  PutPixSafeAvg(x-1,y-1,c2);
               } else {   //If the current pixel is text
                  //If the above pixel is text
                  if ((c2 = GetPixel(x,y-1)) != 255) {
                     c2 = (random(63) + c2)/2;
                     SetPixel(x,y-1,c2);
                  }
                  PutPixSafeAvg(x-1,y-1,c2);
               }
            }
         } else {
            c2 = (GetPixel(x,y-1) + c)/2;
            SetPixel(x,y-1,c2);
            PutPixSafeAvg(x-1,y-1,c2);
         }
      }
   }
   return;
}

void MultiPalette(RGB_color &goal, RGB_color &past, float &progress) {
   float g,c,speed;
   RGB_color current, temp;
   if (progress == 1.0) {
      progress = 0.0;
      Get_Palette_Register(63,past);
      do {
         goal.red = random(63);
         goal.green = random(63);
         goal.blue = random(63);
      } while (goal.red == past.red && goal.green == past.green &&
               goal.blue == past.blue);
   } else {
      current.red = ((float)(goal.red - past.red))*progress + past.red;
      current.green = ((float)(goal.green - past.green))*progress + past.green;
      current.blue = ((float)(goal.blue - past.blue))*progress + past.blue;
      
      for (c=0.0; c < 64.0; c++) {
         g = c/63.0;
         temp.red = current.red*g;
         temp.green = current.green*g;
         temp.blue = current.blue*g;
         Set_Palette_Register(c,temp);
      }
      Get_Palette_Register(63,temp);
      Set_Palette_Register(255,temp);
      
      speed = 1.0/((float)(abs(goal.red-past.red)+abs(goal.green-past.green)+
              abs(goal.blue-past.blue)));
      progress += speed;
      if (progress > 1.0) progress = 1.0;
   }
   return;
}

int Run_Fire_Effect(int textminx, int textminy, int textmaxx, int textmaxy,
                     int flameminx, int flameminy, int flamemaxx, int flamemaxy) {
   float progress=1.0;
   RGB_color goal,past;
   int c,f=0;
   
   randomize();

   Make_Palette();

   long starttime = biostime(_TIME_GETCLOCK, 0)/_BIOS_CLOCKS_PER_SEC;
   
   while (!kbhit()) {
      Wait_For_Vertical_Retrace();
      Flame(textminx,textminy,textmaxx,textmaxy,flameminx,flameminy,
            flamemaxx,flamemaxy, true);
      MultiPalette(goal, past, progress);
      f++;
   }
   long endtime = biostime(_TIME_GETCLOCK, 0)/_BIOS_CLOCKS_PER_SEC;
   PAUSE;

   for (c=0; c < 30; c++) {
      Wait_For_Vertical_Retrace();
      Flame(textminx,textminy,textmaxx,textmaxy,flameminx,flameminy,
            flamemaxx,flamemaxy, false);
   }
   
   return f/(endtime-starttime);
}
